#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## ergnzende Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 09.05.11
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
##
## Um alle Teile des Skripts ausfhren zu knnen, muss das Paket
## mvtnorm samt Abhngigkeiten installiert sein, vgl. Abschnitt 1.2.7:
##
## install.packages("mvtnorm", repos="http://ftp5.gwdg.de/pub/misc/cran/")
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

##### 10.8 Diskriminanzanalyse
## Daten erstellen wie in 10.6.1 - 10.7.1
library(mvtnorm)
sigma <- matrix(c(16,-2, -2,9), byrow=TRUE, ncol=2)
Nj    <- c(15, 25, 20)
mu11  <- c(-4,  4)
mu12  <- c( 3,  3)
mu13  <- c( 1, -1)
Y11   <- round(rmvnorm(n=Nj[1], mean=mu11, sigma=sigma))
Y12   <- round(rmvnorm(n=Nj[2], mean=mu12, sigma=sigma))
Y13   <- round(rmvnorm(n=Nj[3], mean=mu13, sigma=sigma))
Ym1   <- rbind(Y11, Y12, Y13)
IVman <- factor(rep(1:3, Nj))

## Projektionen und Teststatistik-Matrizen wie in 10.8.7
Xb <- cbind(as.numeric(IVman == 1),
            as.numeric(IVman == 2),
            as.numeric(IVman == 3))
Cd <- contr.treatment(ncol(Xb))
Xf <- Xb %*% Cd
X  <- cbind(1, Xf)
Pu <- X  %*% solve(t(X)  %*% X)  %*% t(X)
Xr <- X[ , 1]
Pr <- Xr %*% solve(t(Xr) %*% Xr) %*% t(Xr)
Id <- diag(sum(Nj))
BB <- t(Ym1) %*% (Pu-Pr) %*% Ym1
WW <- t(Ym1) %*% (Id-Pu) %*% Ym1
TT <- t(Ym1) %*% (Id-Pr) %*% Ym1
## fr Daten des Abschnitts diese Datei laden:
# load("data_manovaN.Rdata")

Ydf1 <- data.frame(DV1=Ym1[ , 1], DV2=Ym1[ , 2])

library(MASS)
(ldaRes <- lda(IVman ~ DV1 + DV2, data=Ydf1))
ldaP    <- lda(IVman ~ DV1 + DV2, CV=TRUE, data=Ydf1)
ldaP$posterior

ldaPred <- predict(ldaRes, Ydf1)
head(ldaPred$x)
head(ldaPred$class)

cTab <- table(IVman, ldaPred$class, dnn=c("IVman", "ldaPred"))
addmargins(cTab)
sum(diag(cTab)) / sum(cTab)

eigWinvB <- eigen(solve(WW) %*% BB)
eigVec   <- eigWinvB$vectors
eigVal   <- eigWinvB$values
p        <- nlevels(IVman)
My       <- colMeans(Ym1)
N        <- sum(Nj)

eigVal / sum(diag(solve(WW) %*% BB))
eigVal / sum(eigVal)

scl <- sqrt((N-p) / diag(t(eigVec) %*% WW %*% eigVec))
b0  <- -scl * t(eigVec) %*% My
(bk <- eigVec %*% diag(scl))
ld  <- sweep(Ym1 %*% bk, 2, b0, "+")
all.equal(ld, ldaPred$x, check.attributes=FALSE)

anova(lm(ld[ , 1] ~ IVman))
anova(lm(ld[ , 2] ~ IVman))

((N-p) / (p-1)) * eigVal

priorP <- rep(1/nlevels(IVman), nlevels(IVman))
ldaEq  <- lda(IVman ~ DV1 + DV2, prior=priorP, data=Ydf1)
predEq <- predict(ldaEq, Ydf1)
LDmat  <- predEq$x
ctrDf  <- aggregate(cbind(LD1, LD2) ~ IVman, FUN=mean, data=LDmat)
ctrLD  <- data.matrix(ctrDf[ , -1])

diffMat1 <- sweep(LDmat, 2, ctrLD[1, ], "-")
diffMat2 <- sweep(LDmat, 2, ctrLD[2, ], "-")
diffMat3 <- sweep(LDmat, 2, ctrLD[3, ], "-")
dst1     <- sqrt(diag(tcrossprod(diffMat1)))
dst2     <- sqrt(diag(tcrossprod(diffMat2)))
dst3     <- sqrt(diag(tcrossprod(diffMat3)))
dstMat   <- cbind(dst1, dst2, dst3)
dstPred  <- apply(dstMat, 1, which.min)
head(dstPred)
all(dstPred == unclass(predEq$class))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern mglich
try({
detach(package:mvtnorm)
detach(package:MASS) })
