#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------
## Daniel Wollschlger <dwoll [at] psychologie.uni-kiel.de>
## ergnzende Befehle zum Buch "Grundlagen der Datenanalyse mit R" (Springer)
## http://www.uni-kiel.de/psychologie/dwoll/r/
## Stand: 10.01.12
##
## Vgl. Abschnitt 4.1, um die folgenden Befehle auszufhren.
##
## Um alle Teile des Skripts ausfhren zu knnen, mssen die Pakete
## coin, e1071 samt Abhngigkeiten installiert sein, vgl. Abschnitt 1.2.7:
##
## install.packages(c("coin", "e1071"), repos="http://ftp5.gwdg.de/pub/misc/cran/")
#####---------------------------------------------------------------------------
#####---------------------------------------------------------------------------

####*---------------------------------------------------------------------------
##### 9 Resampling-Verfahren
#####---------------------------------------------------------------------------

##### 9.1 Bootstrapping
##### 9.1.1 Bootstrap-Vertauensintervalle fr mu
muH0 <- 100
sdH0 <- 40
N    <- 200
DV   <- rnorm(N, muH0, sdH0)

getM <- function(orgDV, idx) {
    n     <- length(orgDV[idx])
    bsM   <- mean(orgDV[idx])
    bsS2M <- (((n-1) / n) * var(orgDV[idx])) / n
    c(bsM, bsS2M)
}

library(boot)
nR    <- 999
bsRes <- boot(DV, statistic=getM, R=nR)
## fr Daten des Buches diese Datei laden:
# load("data_bootMu.Rdata")
bsRes

(M      <- mean(DV))
(S2M    <- (((N-1)/N) * var(DV)) / N)
Mstar   <- bsRes$t[ , 1]
S2Mstar <- bsRes$t[ , 2]
(biasM  <- mean(Mstar) - M)
mean(S2Mstar) - S2M
c(sd(Mstar), sd(S2Mstar))

alpha <- 0.05
boot.ci(bsRes, conf=1-alpha, type=c("basic", "perc", "norm", "stud", "bca"))

(idx  <- trunc((nR + 1) * c(alpha/2, 1 - alpha/2)))
tStar <- (Mstar-M) / sqrt(S2Mstar)
tCrit <- sort(tStar)[idx]
zCrit <- qnorm(c(alpha/2, 1 - alpha/2))

(ciBasic <- 2*M - sort(Mstar)[idx])
(ciPerc  <- sort(Mstar)[idx])
(ciNorm  <- M-biasM - zCrit*sd(Mstar))
(ciT     <- M - tCrit*sqrt(S2M))

res <- replicate(nR, getM(DV, sample(seq(along=DV), replace=TRUE)))
## fr Daten des Buches diese Datei laden:
# load("data_bootMu.Rdata")
Mstar  <- res[1, ]
SMstar <- sqrt(res[2, ])
tStar  <- (Mstar-mean(DV)) / SMstar

dev.new(width=7, height=7)
plot(tStar, ecdf(tStar)(tStar), col="gray60", pch=1, xlab="t* bzw. t",
     ylab="P(T <= t)", main="t*: Kumulierte rel. Hufigkeiten und Verteilungsfunktion")
curve(pt(x, N-1), lwd=2, add=TRUE)
legend(x="topleft", lty=c(NA, 1), pch=c(1, NA), lwd=c(2, 2),
       col=c("gray60", "black"), legend=c("t*", "t"))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern mglich
try(detach(package:boot))

##### 9.1.2 Bootstrap-Vertrauensintervalle fr Regressionsparameter
## Daten erstellen wie in 7.3
N      <- 100
height <- rnorm(N, 175, 7)
age    <- rnorm(N,  30, 8)
sport  <- abs(rnorm(N, 60, 30))
weight <- 0.5*height - 0.3*age - 0.4*sport + 10 + rnorm(N, 0, 3)
fitHAS <- lm(weight ~ height + age + sport)
regrDf <- data.frame(weight, height, age, sport)
## fr Daten des Buches diese Datei laden:
# load("data_multRegrN.Rdata")

sqrt(diag(vcov(fitHAS)))
confint(fitHAS)

getRegr <- function(dat, idx) {
    bsFit <- lm(weight ~ height + age + sport, subset=idx, data=dat)
    coefficients(bsFit)
}

library(boot)
nR <- 999
(bsRegr <- boot(regrDf, statistic=getRegr, R=nR))

boot.ci(bsRegr, conf=0.95, type="bca", index=1)$bca
boot.ci(bsRegr, conf=0.95, type="bca", index=2)$bca
boot.ci(bsRegr, conf=0.95, type="bca", index=3)$bca
boot.ci(bsRegr, conf=0.95, type="bca", index=4)$bca

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern mglich
try(detach(package:boot))

##### 9.1.3 Bootstrapping in varianzanalytischen Design
## fr Daten des Buches diese Datei laden:
# load("data_bootAnova.Rdata")
P     <- 4
Nj    <- c(41, 37, 42, 40)
DVa   <- rnorm(Nj[1], 0,   1)
DVb   <- rnorm(Nj[2], 0.3, 1)
DVc   <- rnorm(Nj[3], 0.6, 1)
DVd   <- rnorm(Nj[4], 1.0, 1)
DV    <- c(DVa, DVb, DVc, DVd)
IV    <- factor(rep(LETTERS[1:P], Nj))
dfCRp <- data.frame(IV, DV)
## fr Daten des Buches diese Datei laden:
# load("data_bootAnova.Rdata")
anBase <- anova(lm(DV ~ IV))
Fbase  <- anBase["IV", "F value"]
(pBase <- anBase["IV", "Pr(>F)"])

fit0   <- lm(DV ~ 1)
E      <- residuals(fit0)
Er     <- E / sqrt(1-hatvalues(fit0))
Yhat   <- fitted(fit0)

getAnova <- function(dat, idx) {
    Ystar <- Yhat + Er[idx]
    anBS  <- anova(lm(Ystar ~ IV, data=dat))
    anBS["IV", "F value"]
}

library(boot)
nR       <- 999
(bsAnova <- boot(dfCRp, statistic=getAnova, R=nR))
Fstar    <- bsAnova$t
(pValBS  <- (sum(Fstar >= Fbase) + 1) / (length(Fstar) + 1))

dev.new(width=7, height=7)
plot(Fstar, ecdf(Fstar)(Fstar), col="gray60", pch=1, xlab="f* bzw. f",
     ylab="P(F <= f)", main="F*: Kumulierte rel. Hufigkeiten und Verteilungsfunktion")
curve(pf(x, P-1, sum(Nj) - P), lwd=2, add=TRUE)
legend(x="topleft", lty=c(NA, 1), pch=c(1, NA), lwd=c(2, 2),
       col=c("gray60", "black"), legend=c("F*", "F"))

getAnovaWild <- function(dat, idx) {
    n  <- length(idx)
    Ur <- sample(c(-1, 1), size=n, replace=TRUE, prob=c(0.5, 0.5))
    Uf <- sample(c(-(sqrt(5) - 1)/2, (sqrt(5) + 1)/2), size=n, replace=TRUE,
                 prob=c((sqrt(5) + 1)/(2*sqrt(5)), (sqrt(5) - 1)/(2*sqrt(5))))

    Ystar <- Yhat + (Er*Ur)[idx]
    # Ystar <- Yhat + (Er*Uf)[idx]
    anBS  <- anova(lm(Ystar ~ IV, data=dat))
    anBS["IV", "F value"]
}

bsAnovaW <- boot(dfCRp, statistic=getAnovaWild, R=nR)
(FstarW  <- bsAnovaW$t)
(pValBSw <- (sum(FstarW >= Fbase) + 1) / (length(FstarW) + 1))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern mglich
try(detach(package:boot))

##### 9.2 Permutationstests
##### 9.2.1 Test auf gleiche Lageparameter in unabhngigen Stichproben
Nj    <- c(7, 8)
DVa   <- round(rnorm(Nj[1], 100, 20))
DVb   <- round(rnorm(Nj[2], 110, 20))
## fr Daten des Buches diese Datei laden:
# load("data_permTest.Rdata")
DVab  <- c(DVa, DVb)
IVbtw <- factor(rep(c("A", "B"), Nj))

library(coin)
oneway_test(DVab ~ IVbtw, alternative="less", distribution="exact")
t.test(DVab ~ IVbtw, alternative="less", var.equal=TRUE)$p.value

idx   <- seq(along=DVab)
idxA  <- combn(idx, Nj[1])
getDM <- function(x) { mean(DVab[x]) - mean(DVab[!(idx %in% x)]) }
resDM <- apply(idxA, 2, getDM)
diffM <- mean(DVa) - mean(DVb)
(pVal <- sum(resDM <= diffM) / length(resDM))

dev.new(width=12, height=6.5)
op <- par(mfrow=c(1, 2), cex.lab=1.2, cex.main=1.2)
hist(resDM, freq=FALSE, breaks="FD", xlab="Mittelwertsdifferenzen",
     main="Permutationstest: Histogramm Mittelwertsdifferenzen")
curve(dnorm(x, mean(resDM), sd(resDM)), lwd=2, add=TRUE)
legend(x="topright", lty=1, lwd=2, legend=expression(paste("N(M, ", s^2, ")")))

plot(resDM, ecdf(resDM)(resDM), col="gray60", pch=16,
     xlab="Mittelwertsdifferenzen", ylab="kumulierte relative Hufigkeit",
     main="Kumulierte relative Hufigkeiten und Verteilungsfunktion")
curve(pnorm(x, mean(resDM), sd(resDM)), lwd=2, add=TRUE)
legend(x="bottomright", lty=c(NA, 1), pch=c(16, NA), lwd=c(1, 2),
       col=c("gray60", "black"),
       legend=c("Permutationen", expression(paste("N(M, ", s^2, ")"))))
par(op)

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern mglich
try({
detach(package:coin)
detach(package:modeltools)
detach(package:survival)
detach(package:mvtnorm)
detach(package:splines)
detach(package:stats4) })

##### 9.2.2 Test auf gleiche Lageparameter in abhngigen Stichproben
N      <- 12
id     <- factor(rep(1:N, times=2))
DVpre  <- round(rnorm(N, 100, 20))
DVpost <- round(rnorm(N, 110, 20))
## fr Daten des Buches diese Datei laden:
# load("data_permTest.Rdata")
DVpp   <- c(DVpre, DVpost)
IV     <- factor(rep(0:1, each=N), labels=c("pre", "post"))

library(coin)
oneway_test(DVpp ~ IV | id, alternative="less", distribution=approximate(B=9999))

t.test(DVpp ~ IV, alternative="less", paired=TRUE)$p.value

DVd    <- DVpre-DVpost
sgnLst <- lapply(numeric(N), function(x) { c(-1, 1) } )
sgnMat <- data.matrix(expand.grid(sgnLst))
getMD  <- function(x) { mean(abs(DVd) * x) }
resMD  <- apply(sgnMat, 1, getMD)
(pVal  <- sum(resMD <= mean(DVd)) / length(resMD))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern mglich
try({
detach(package:coin)
detach(package:modeltools)
detach(package:survival)
detach(package:mvtnorm)
detach(package:splines)
detach(package:stats4) })

##### 9.2.3 Test auf Unabhngigkeit von zwei Variablen
Nf  <- 7
DV1 <- rbinom(Nf, size=1, prob=0.5)
DV2 <- rbinom(Nf, size=1, prob=0.5)
## fr Daten des Buches diese Datei laden:
# load("data_permTest.Rdata")
fisher.test(DV1, DV2, alternative="greater")$p.value

library(e1071)
permIdx <- permutations(Nf)

getAgree <- function(idx) {
    sum(diag(table(DV1, DV2[idx])))
}

resAgree <- apply(permIdx, 1, getAgree)
agree12  <- sum(diag(table(DV1, DV2)))
(pVal    <- sum(resAgree >= agree12) / length(resAgree))

## entferne die (ggf. automatisch) geladenen Zusatzpakete, sofern mglich
try({
detach(package:e1071)
detach(package:class) })
